@@ -23,7 +23,9 @@ BENCHMARK_DECLARE (sizes)
 BENCHMARK_DECLARE (loop_count)
 BENCHMARK_DECLARE (loop_count_timed)
 BENCHMARK_DECLARE (ping_pongs)
-BENCHMARK_DECLARE (ping_udp)
+BENCHMARK_DECLARE (ping_udp1)
+BENCHMARK_DECLARE (ping_udp10)
+BENCHMARK_DECLARE (ping_udp100)
 BENCHMARK_DECLARE (tcp_write_batch)
 BENCHMARK_DECLARE (tcp4_pound_100)
 BENCHMARK_DECLARE (tcp4_pound_1000)
@@ -72,6 +74,7 @@ BENCHMARK_DECLARE (async_pummel_1)
 BENCHMARK_DECLARE (async_pummel_2)
 BENCHMARK_DECLARE (async_pummel_4)
 BENCHMARK_DECLARE (async_pummel_8)
+BENCHMARK_DECLARE (queue_work)
 BENCHMARK_DECLARE (spawn)
 BENCHMARK_DECLARE (thread_create)
 BENCHMARK_DECLARE (million_async)
@@ -90,6 +93,10 @@ TASK_LIST_START
   BENCHMARK_ENTRY  (ping_pongs)
   BENCHMARK_HELPER (ping_pongs, tcp4_echo_server)
 
+  BENCHMARK_ENTRY  (ping_udp1)
+  BENCHMARK_ENTRY  (ping_udp10)
+  BENCHMARK_ENTRY  (ping_udp100)
+
   BENCHMARK_ENTRY  (tcp_write_batch)
   BENCHMARK_HELPER (tcp_write_batch, tcp4_blackhole_server)
 
@@ -155,6 +162,7 @@ TASK_LIST_START
   BENCHMARK_ENTRY  (async_pummel_2)
   BENCHMARK_ENTRY  (async_pummel_4)
   BENCHMARK_ENTRY  (async_pummel_8)
+  BENCHMARK_ENTRY  (queue_work)
 
   BENCHMARK_ENTRY  (spawn)
   BENCHMARK_ENTRY  (thread_create)
